// 0.7.1:
// 0.7:
//   - Cleaned up .css
//   - Moved all event handlers into .xml
//   - Added translations for Chinese and Polish
//   - Remove schedule UI (menu item, panel, translations, event bindings)
//   - Tested in Firefox 1.0
// 0.6:
//   - Fixed the search box by having textbox-nav-tree bindings extend
//     #timed-textbox instead of just #textbox.
//   - Added annoying rollover text to statusbar to help diminish the
//     even more annoying complaints from people who couldn't find the
//     View menu.
//   - Toolbar buttons now work regardless of where the user focus is.
//   - Changed .css so that description is multiline again.
//   - Use DTD entities to allow for future i18n
// 0.5:
//   Repackaged for Firefox 0.9
// 0.4:
//   Regular preference dialog now works again.
//   When using arrow keys to change bookmarks while focus is in a
//   text field, if that field became disabled, you'd get "stuck".
//   Focus is now adjusted and replaced to avoid getting stuck.
//   Fixed the menuitem checkmarks for the toolbars.

// TODO:
//   - Fix updateURL install stuff
//   - Compare to similar code in latest firefox; bring over any bug fixes
//     - Add feed location functionality
//   - Review email for other bug reports
//   - Make sure arrow keys move selection with focus on checkbox and
//     text fields
//   - Put more static text in locale files to be translatable
//   - Add help button in toolbar -- across from "sidebar" checkbox?
//   - Add plugin name and version number as well?
//   - Add key bindings so ctrl-arrow always moves selection, even in
//   multi-line description.
//   - Clean up extra statusbar
//   - Now that there's only one panel, hide extra status line at the
//   same time as the panel?
//   - Fix tabbing when panel is closed in Windows.
//   - Changes in regular Properties dialog should not be lost.
//   - The extension SortBookmarks allows to sort "folders first". But
//   this does not effect the bookmark manager.


// This is the RDF Resource we're dealing with.
var gResource;
// This is the set of fields that are visible in the window.
var gFields;
// ...and this is a parallel array that contains the RDF properties
// that they are associated with.
var gProperties;

var gfbAutosave = true;

var gfbHoldFocus = false;
var gfbFocus = false;

// add a toolbar toggle to the bmark manager's View menu
window.addEventListener("load", fbmInit, false);

function list( str, obj, detail )
{
  str += ":\n";
  for( i in obj ) {
    str += i;
    if( detail )
      str += " = " + obj[i];
    str += ";  ";
  }
  alert(str);
}

function fbmUpdateAttribute(aProperty, aOldValue, aNewValue)
{
  if ((aOldValue || aNewValue) && aOldValue != aNewValue) {
    if (aOldValue && !aNewValue)
      BMDS.Unassert(gResource, aProperty, aOldValue);
    else if (!aOldValue && aNewValue)
      BMDS.Assert(gResource, aProperty, aNewValue, true);
    else /* if (aOldValue && aNewValue) */
      BMDS.Change(gResource, aProperty, aOldValue, aNewValue);
    return true;
  }
  return false;
}

// largely scammed from browser/bookmarks/bookmarksProperties.js
function treeOnSelect()
{
  if( ! gFields ) {
    // This is the set of fields that are visible in the window.
    gFields = ["name", "url", "shortcut", "description", "webpanel", "url" ];

    // ...and this is a parallel array that contains the RDF properties
    // that they are associated with.
    gProperties = [RDF.GetResource(gNC_NS+"Name"),
                   RDF.GetResource(gNC_NS+"URL"),
                   RDF.GetResource(gNC_NS+"ShortcutURL"),
                   RDF.GetResource(gNC_NS+"Description"),
                   RDF.GetResource(gNC_NS+"WebPanel"),
                   RDF.GetResource(gNC_NS+"FeedURL")];
  }

  // If there was something selected previously, save it off quick...
  FBMSave();

  // Then do basically what the Properties window does, but with our
  // own details...

  gfbAutosave = false;
  var item = document.getElementById("bookmarks-view")._selection.item;
  var areMultiple = item.length > 1;
  if( item.length > 0 && item[0] )
    gResource = RDF.GetResource(item[0].Value);

  var bookmarkType = BookmarksUtils.resolveType(gResource);
  var isLivemark = bookmarkType == "Livemark";
  var isSeparator = bookmarkType == "BookmarkSeparator";

  // Initialize the properties panel by copying the values from the
  // RDF graph into the fields on screen.

  document.getElementById("url").value = "";

  for (var i=0; i<gFields.length; ++i) {
    var field = document.getElementById(gFields[i]);
    var value = BMDS.GetTarget(gResource, gProperties[i], true);

    if (value)
      value = value.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;

    if (gFields[i] == "webpanel")
      field.checked = (value != undefined);
    else if (gFields[i] != "url" || value)
      field.value = value;
  }

  // if its a container, disable some things
  var isContainerFlag = RDFCU.IsContainer(BMDS, gResource);
  if (!isContainerFlag) {
    // XXX To do: the "RDFCU.IsContainer" call above only works for
    // RDF sequences; if its not a RDF sequence, we should to more
    // checking to see if the item in question is really a container
    // of not.  A good example of this is the "File System" container.
  }

  var disable = (isContainerFlag || isSeparator || areMultiple || isLivemark);

/* I don'e know how to refer to an entity from JavaScript.
  if( isLivemark )
    document.getElementById("urllabel").value = "&bookmarks.feedurl.label;"
  else
    document.getElementById("urllabel").value = "&bookmarks.location.label;"
*/

  // Disable various fields based on current selection
  document.getElementById("name"       ).disabled = areMultiple;
  document.getElementById("url"        ).disabled = disable && ! isLivemark;
  document.getElementById("shortcut"   ).disabled = disable;
  document.getElementById("description").disabled = isSeparator || areMultiple;
  document.getElementById("webpanel"   ).disabled = disable;

  // Keep focus in a field that isn't disabled
  if( gfbFocus ) {
    if( gfbFocus.disabled ) {
      gfbHoldFocus = true;
      document.getElementById("name").focus();
      gfbHoldFocus = false;
    }
    else {
      gfbFocus.focus();
    }
  }

  gfbAutosave = true;
}

function FBMSave()
{
  var changed = false;
  if( ! gResource || ! gfbAutosave )
    return;

  var bookmarkType = BookmarksUtils.resolveType(gResource);
  var isLivemark = bookmarkType == "Livemark";

  // Grovel through the fields to see if any of the values have
  // changed. If so, update the RDF graph and force them to be saved
  // to disk.
  for (var i=0; i<gFields.length; ++i) {
    var field = document.getElementById(gFields[i]);

    if (! field)
      continue;

    if (gFields[i] == "url" && (
        (isLivemark && i == 1) || (!isLivemark && i != 1) ) )
      continue;

    // Get the new value as a literal, using "null" if the value is empty.
    var newValue;
    if (gFields[i] == "webpanel")
      newValue = field.checked ? "true" : undefined;
    else
      newValue = field.value;

    var oldValue = BMDS.GetTarget(gResource, gProperties[i], true);

    if (oldValue)
      oldValue = oldValue.QueryInterface(Components.interfaces.nsIRDFLiteral);

    if (newValue && gFields[i] == "shortcut")
      // shortcuts are always lowercased internally
      newValue = newValue.toLowerCase();
    else if (newValue && gFields[i] == "url") {
      if (newValue.indexOf(":") < 0)
        // we're dealing with the URL attribute;
        // if a scheme isn't specified, use "http://"
        newValue = "http://" + newValue;
    }

    if (newValue)
      newValue = RDF.GetLiteral(newValue);

    changed |= fbmUpdateAttribute(gProperties[i], oldValue, newValue);
  }

  if (changed) {
    var remote = BMDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    if (remote)
      remote.Flush();
  }

  return true;
}

function fbmFocus(obj)
{
  if( ! gfbHoldFocus )
    gfbFocus = obj;
}

function fbmBlur(obj)
{
  if( ! gfbHoldFocus )
    gfbFocus = false;
}

function fbmFieldChange(obj)
{
  FBMSave();
}

function treeSelect( event, delta )
{
  var bo = document.getElementById("bookmarks-view").treeBoxObject;
  var i = bo.selection.currentIndex + delta;
  if( 0 <= i && i < bo.view.rowCount ) {
    bo.selection.select( i );
    bo.ensureRowIsVisible( i );
    event.preventBubble();
  }
}

function fbmInit()
{
  // Move menuitems to toggle the Properties toolbar
  // into the View menu
  var fillhere = document.getElementById("descending");
  if( ! fillhere ) return;  // Used when opening bookmark properties

  var vp = document.getElementById("viewPropsToolbar");
  var hid = document.getElementById("flatbmark-props").hidden;
  vp.parentNode.removeChild( vp );
  vp.setAttribute("checked", !hid);
  vp.setAttribute("hidden", false);
  fillhere.parentNode.insertBefore( vp, fillhere );

  // allow the toolbar buttons to work when focus is anywhere in the manager
  var ctrl = document.getElementById("bookmarks-view").controller;
  window.controllers.appendController( ctrl );

  var tree = document.getElementById( "bookmarks-view" );
  tree.addEventListener("select", treeOnSelect, false);
  treeOnSelect(); // once to get things started
}
